"""
安全模块工具
"""
from datetime import datetime, timedelta, timezone
from bcrypt import checkpw, gensalt, hashpw
from base64 import b64decode, b64encode
from Cryptodome.Cipher import AES
from Cryptodome.Util.Padding import pad, unpad
from jose import jwt


class AESAPI:
    def __init__(self, key: str) -> None:
        # ECB模式,加密结果固定
        self._cipher = AES.new(key.encode(), AES.MODE_ECB)

    @property
    def cipher(self):
        return self._cipher

    def encrypt(self, plain_text: str) -> str:
        """加密:pad->encrypt->b64encode
        """

        # 对明文进行补全至块大小的填充
        padded_text = pad(plain_text.encode(), AES.block_size)

        # 加密
        ciphertext = self.cipher.encrypt(padded_text)

        # 返回Base64编码的密文
        return b64encode(ciphertext).decode()

    def decrypt(self, encrypted_value: str) -> str:
        """解密:b64decode->decrypt->unpad
        """

        # 将Base64编码的密文解码为字节
        ciphertext = b64decode(encrypted_value)

        # 解密
        decrypted_text = unpad(self.cipher.decrypt(ciphertext), AES.block_size)

        # 将解密后的字节串转换回字符串
        return decrypted_text.decode()

    def phone_encrypt(self, plain_text: str) -> str:
        """加密手机号;手机号按照每3位组成一段密文,然后拼接而成;
        如:18012345678,分为 180,801,012,123,......678;
        使用AES ECB模式加密,保障同样的明文输出同样的密文,用于手机号的模糊匹配

        Args:
            plain_text:手机号明文,例如:18012345678

        Returns:
            str:加密后的手机号
        """
        if (length := len(plain_text)) == 0:
            return ""

        end_pos = max(length-3, 0)
        return ",".join([self.encrypt(plain_text[i:i+3]) for i in range(end_pos+1)])

    def phone_decrypt(self, encrypted_text: str, mask: bool = True) -> str:
        """解密手机号

        Args:
            encrypted_text:加密的手机号
            mask: 是否脱敏显示,比如180****5678        

        Returns:
            str:手机号明文
        """
        if not encrypted_text:
            return ""

        phone_array = [self.decrypt(v) for v in encrypted_text.split(",")]
        phone = "".join([phone_array[i][0]
                        for i in range(8)]) + phone_array[-1]

        if mask:
            phone = f"{phone[:3]}****{phone[7:]}"

        return phone


class HashAPI:
    def hash(self, plain_text: str) -> str:
        """明文哈希加密
        """
        return hashpw(plain_text.encode(), gensalt()).decode()

    def verify(self, plain_text: str, hash_text: str) -> bool:
        """验证明文和密文的内容是否一致
        """
        return checkpw(plain_text.encode(), hash_text.encode())


class JWTAPI:
    def __init__(self, key: str, expire_min: int, algorithm: str = "HS256") -> None:
        """JWT对象

        Args:
            key:str,密钥
            expire_min:int,token的有效期(分钟)
            algorithm:str,加密算法

        """
        self.__key = key
        self.__algorithm = algorithm
        self.expire = expire_min

    @property
    def key(self):
        return self.__key

    @property
    def algorithm(self):
        return self.__algorithm

    def encode(self, **kw) -> str:
        """编码JWT的token
        """
        expire = datetime.now(timezone.utc) + timedelta(minutes=self.expire)
        # expire = datetime.now(ZoneInfo("UTC")) + timedelta(minutes=self.expire)
        payload = dict(**kw, exp=expire)
        return jwt.encode(payload, key=self.key, algorithm=self.algorithm)

    def decode(self, token: str) -> dict:
        """解码JWT的token
        """
        return jwt.decode(token=token, key=self.key, algorithms=[self.algorithm])


if __name__ == "__main__":
    ...



"""
作者:J.Hu
日期:2023-12-19
内容:
# 功能描述：
 - 支持邮件发送
# 重要版本变更说明：
 - 更新附件的代码编写,使其支持正常的文字显示;
 - To/Cc/Bcc的列表拼接采用逗号的形式拼接,原因是原来采用多个To/Cc/Bcc会导致
   在某些邮件客户端上只显示一个To/Cc/Bcc内容.
   比如,Header里面内容如:
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   这样只会显示一个huj,
   现在修改变成：
   To:=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>,=?utf-8?b?6IOh5YGl?= <huj@citytsm.com>
   这样就可以显示出2个huj,huj了
"""
from smtplib import SMTP, SMTP_SSL
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.utils import formataddr
from email.header import Header


class EmailSender:
    def __init__(self, host: str, port: int = 465, ssl: bool = True, timeout: int = 10, encode: str = "utf-8", mail_type: str = "html", attach_type: str = "base64") -> None:
        """Email对象

        Args:
            host:str,STMP的服务器地址
            port:int,STMP的端口号,默认465
            ssl:bool,是否使用ssl,默认true
            timeout:int,服务器连接超时(秒)
            encode:str,邮件编码,默认为utf-8
            mail_type:str,邮件正文类型,默认为html
            attch_type:str,邮件附件类型,默认为base64
        """
        # 根据SSL确定SMTP的类型
        smtpfunc = SMTP_SSL if ssl else SMTP
        self.__smtp = smtpfunc(host, port, timeout=timeout)

        self.encode = encode
        self.mail_type = mail_type
        self.attach_type = attach_type

        # 初始化to,cc,bcc对象

    @property
    def smtp(self):
        return self.__smtp

    @property
    def sender(self):
        return self.__sender

    def login(self, acct: str, passwd: str, nickname: str | None = None) -> None:
        """登录SMTP服务器

        Args:
            acct:str,账号,比如:joestarhu@163.com
            passwd:str,密码
            nickname:str,昵称
        """
        self.smtp.login(acct, passwd)
        if not nickname:
            nickname = acct
        self.__sender = formataddr([nickname, acct], self.encode)
        self.acct = acct

    def send(self, title: str, content: str, attach_list: list[str] = [], to: list[str] | None = None, cc: list[str] | None = None, bcc: list[str] | None = None) -> None:
        """发送邮件

        Args:
            title:str,邮件正文标题
            content:str,邮件正文内容
            attach_list:list,邮件附件列表, ['/user/a.zip','/user/b.zip']
            to:list,收件人列表,['a@a.com','b@b.com']
            cc:list,cc列表,['a@a.com','b@b.com']
            bcc:list,bcc列表,['a@a.com','b@b.com']
        """
        mime = MIMEMultipart()

        # 设置邮件标题和发件人
        mime["Subject"] = Header(title, self.encode)
        mime["From"] = self.sender

        # 设置收件人信息 to,cc,bcc
        receive_lst = []
        rcpt_to = dict(label="To", value=to)
        rcpt_cc = dict(label="Cc", value=cc)
        rcpt_bcc = dict(label="Bcc", value=bcc)

        for val in [rcpt_to, rcpt_cc, rcpt_bcc]:
            # 没有设置相关的收件人信息
            rcpt_type = val["label"]
            rcpt_value = val["value"]

            if not rcpt_value:
                continue
            receive_lst.extend(rcpt_value)
            mime[rcpt_type] = ','.join(
                [formataddr([None, mail], self.encode) for mail in rcpt_value])

        # 正文内容设置
        mime.attach(MIMEText(content, self.mail_type, self.encode))

        # 附件内容设置
        for idx, val in enumerate(attach_list):
            att = MIMEText(open(val, "rb").read(), self.mail_type, self.encode)
            att.add_header("Content-ID", str(idx))
            att.add_header("Content-Type", "application/octet-stream")
            att_name = val.split('/')[-1]
            att.add_header("Content-Disposition", "attachment",
                           filename=(self.encode, '', att_name))
            mime.attach(att)

        # 发送邮件
        self.smtp.sendmail(self.acct, receive_lst, mime.as_string())

    def close(self):
        """关闭smtp链接
        """
        self.smtp.close()


if __name__ == "__main__":
    host = "host"
    e = EmailSender(host)
    acct = "a@a.com"
    passwd = "passwd"
    nickname = "joestarhu"
    e.login(acct, passwd, nickname)
    e.send(title="title", content="content", to=["a@a.com", "b@b.com"])



"""
整合钉钉,飞书第三方网页登录
"""
from enum import IntEnum
from urllib.parse import quote_plus
from requests import post, get


class AuthType(IntEnum):
    # 钉钉
    DINGTALK = 0
    # 飞书
    FEISHU = 1


def get_dingtalk_user_info(ak: str, sk: str, auth_code: str) -> dict:
    """获取钉钉用户信息

    Args:
        ak: 钉钉应用的AppKey, 如:ding1cincjju0rvrl6c5
        sk: 钉钉应用的SecurityKey,如:w4dkdOyB3iy9lMv7VstUqnVHgj3f-WmbUGiHj2X-HsE4Yp_Fn0pHrQ_deI1oy1Nl
        auth_code: 钉钉扫码得到的auth_code, 如:e8ce805bbd5434168292f75c11832eac
    """
    # 1.获取access_token
    ACCESS_TOKEN_URL = "https://api.dingtalk.com/v1.0/oauth2/userAccessToken"
    data = {"clientId": ak, "clientSecret": sk,
            "code": auth_code, "grantType": "authorization_code"}

    with post(ACCESS_TOKEN_URL, json=data) as rsp:
        access_token = rsp.json().get("accessToken", "")

    # 2.获取用户信息
    USER_INFO_URL = "https://api.dingtalk.com/v1.0/contact/users/me"
    headers = {"x-acs-dingtalk-access-token": f"{access_token}"}

    with get(USER_INFO_URL, headers=headers) as rsp:
        result = rsp.json()

    return result


def get_feishu_user_info(ak: str, sk: str, auth_code: str) -> dict:
    """获取飞书的用户信息

    Args:
        ak: 飞书应用的AppKey,如:cli_3a2c007d00da5bdc
        sk: 飞书应用的SecurityKey,如:BZUG0yREh8XJfl14teRZ6rr0HeFhr2I5
        auth_code: 飞书扫码得到的auth_code , 如:5c0t8bf1efe14119a00c5555ee066053
    """
    # 1.获取app_access_token
    app_access_token_data = {"app_id": ak, "app_secret": sk}
    APP_ACCESS_TOKEN_URL = "https://open.feishu.cn/open-apis/auth/v3/app_access_token/internal"
    with post(APP_ACCESS_TOKEN_URL, json=app_access_token_data) as rsp:
        app_access_token = rsp.json().get("app_access_token", "")

    # 2.获取access_token
    ACCESS_TOKEN_URL = "https://open.feishu.cn/open-apis/authen/v1/oidc/access_token"
    access_token_headers = {"Authorization": f"Bearer {app_access_token}"}
    access_token_data = {"grant_type": "authorization_code", "code": auth_code}
    with post(ACCESS_TOKEN_URL, headers=access_token_headers, json=access_token_data) as rsp:
        user_access_token = rsp.json()["data"]["access_token"]

    # 3.获取用户信息
    USER_INFO_URL = "https://open.feishu.cn/open-apis/authen/v1/user_info"
    user_access_token_headers = {
        "Authorization": f"Bearer {user_access_token}"}
    with get(USER_INFO_URL, headers=user_access_token_headers) as rsp:
        result = rsp.json()
    return result


class ThridAuthBase:
    def __init__(self, ak: str, sk: str, auth_type: AuthType) -> None:
        """三方扫码登录初始化

        Args:
            ak:str,应用的appkey
            sk:str,应用的secretkey
            auth_type:AuthType,应用类型,当前支持 钉钉:AuthType.DINGTALK,飞书:AuthType.FEISHU
        """
        self._auth_type = auth_type
        self._ak = ak
        self._sk = sk

        # 回调函数注册
        match(auth_type):
            case AuthType.DINGTALK:
                # 钉钉三方登录网页地址拼接
                self._prefix_url = "https://login.dingtalk.com/oauth2/challenge.htm"
                self.get_user_info_fn = get_dingtalk_user_info
            case AuthType.FEISHU:
                # 飞书三方登录网页地址拼接
                self._prefix_url = "https://open.feishu.cn/open-apis/authen/v1/authorize"
                self.get_user_info_fn = get_feishu_user_info
            case _:
                raise TypeError("认证类型类型错误")

    @property
    def ak(self) -> str:
        return self._ak

    @property
    def sk(self) -> str:
        return self._sk

    @property
    def auth_type(self) -> AuthType:
        return self._auth_type

    @property
    def prefix_url(self) -> str:
        return self._prefix_url

    def create_login_url(self, redirect_uri: str, state: str | None = None) -> str:
        """生成三方扫码登录的url地址

        Args:
            redirect_uri: 扫码通过后,回调的地址
            state: 生成地址用的state

        Returns:
            三方扫码登录的url地址
        """

        match(self.auth_type):
            case AuthType.DINGTALK:
                state = state or "JHU_DINGTALK"
                data = {
                    "redirect_uri": quote_plus(redirect_uri),
                    "client_id": self.ak,
                    "response_type": "code",
                    "scope": "openid",
                    "state": state,
                    "prompt": "consent"
                }
            case AuthType.FEISHU:
                state = state or "JHU_FEISHU"
                data = {
                    "redirect_uri": quote_plus(redirect_uri),
                    "app_id": self.ak,
                    "scope": "contact:user.phone:readonly",
                    "state": state
                }
            case _:
                raise TypeError("认证类型类型错误")

        query_params = '&'.join([f"{k}={data[k]}" for k in data])
        url = self.prefix_url+'?'+query_params
        return url

    def get_user_info(self, auth_code: str) -> dict:
        """获取三方扫码得到的用户信息

        Args:
            auth_code:str,三方登录连接返回的auth_code

        Returns:
            用户的json信息
        """
        return self.get_user_info_fn(self.ak, self.sk, auth_code)


class DingTalk(ThridAuthBase):
    def __init__(self, ak: str, sk: str) -> None:
        """钉钉三方登录

        Args:
            ak: 钉钉应用的AppKey, 如:ding1cincjju0rvrl6c5
            sk: 钉钉应用的SecurityKey,如:w4dkdOyB3iy9lMv7VstUqnVHgj3f-WmbUGiHj2X-HsE4Yp_Fn0pHrQ_deI1oy1Nl
        """
        super().__init__(ak, sk, AuthType.DINGTALK)


class FeiShu(ThridAuthBase):
    def __init__(self, ak: str, sk: str) -> None:
        """飞书三方登录

        Args:
            ak: 飞书应用的AppKey,如:cli_3a2c007d00da5bdc
            sk: 飞书应用的SecurityKey,如:BZUG0yREh8XJfl14teRZ6rr0HeFhr2I5
        """
        super().__init__(ak, sk, AuthType.FEISHU)


if __name__ == "__main__":
    ding_ak = "ding_ak"
    ding_sk = "ding_sk"
    d = DingTalk(ding_ak, ding_sk)
    d.create_login_url("http://localhost:9000/")
    # 这里的code从redirect_url返回的code中获取
    d.get_user_info("968999a21b9b38c794d230c5d80742bc")

    feishu_ak = "feishu_ak"
    feishu_sk = "feishu_sk"
    f = FeiShu(feishu_ak, feishu_sk)
    f.create_login_url("http://localhost:9000/")
    # 这里的code从redirect_url返回的code中获取
    f.get_user_info("b8ag458bd0f54712be8c6877273ce415")
